#ifndef MSP_GL_TEXTURE1D_H_
#define MSP_GL_TEXTURE1D_H_

#include "texture.h"

namespace Msp {
namespace GL {

class Texture1D: public Texture
{
public:
	class Loader: public DataFile::DerivedObjectLoader<Texture1D, Texture::Loader>
	{
	public:
		Loader(Texture1D &);
		Loader(Texture1D &, Collection &);
	private:
		void init();

		void raw_data(const std::string &);
		void storage(PixelFormat, unsigned);
		void storage_levels(PixelFormat, unsigned, unsigned);
	};

private:
	unsigned width;
	unsigned levels;
	unsigned allocated;

public:
	Texture1D();

	void storage(PixelFormat, unsigned, unsigned = 0);

	DEPRECATED void storage(PixelComponents c, unsigned w, unsigned l = 0)
	{ storage(make_pixelformat(c, UNSIGNED_BYTE), w, l); }

	void allocate(unsigned);
	void image(unsigned, const void *);
	DEPRECATED void image(unsigned, PixelComponents, DataType, const void *);
	void sub_image(unsigned, int, unsigned, const void *);
	DEPRECATED void sub_image(unsigned, int, unsigned, PixelComponents, DataType, const void *);
	virtual void image(const Graphics::Image &, unsigned = 0);
	using Texture::image;
	unsigned get_width() const { return width; }

private:
	unsigned get_n_levels() const;
	unsigned get_level_size(unsigned) const;

public:
	virtual AsyncLoader *load(IO::Seekable &, const Resources * = 0) { return 0; }
	virtual UInt64 get_data_size() const;
	virtual void unload() { }
};

} // namespace GL
} // namespace Msp

#endif
